local __exports = LibStub:NewLibrary("ovale/states/runeforge", 90104)
if not __exports then return end
local __class = LibStub:GetLibrary("tslib").newClass
local aceEvent = LibStub:GetLibrary("AceEvent-3.0", true)
local kpairs = pairs
local pairs = pairs
local tonumber = tonumber
local concat = table.concat
local insert = table.insert
local C_LegendaryCrafting = C_LegendaryCrafting
local Enum = Enum
local __enginecondition = LibStub:GetLibrary("ovale/engine/condition")
local returnBoolean = __enginecondition.returnBoolean
__exports.Runeforge = __class(nil, {
    constructor = function(self, ovale, debug, equipment)
        self.equipment = equipment
        self.equippedLegendaryById = {}
        self.equippedRuneforgeById = {}
        self.debugRuneforges = {
            type = "group",
            name = "Runeforges",
            args = {
                runeforge = {
                    type = "input",
                    name = "Runeforges",
                    multiline = 25,
                    width = "full",
                    get = function()
                        local ids = C_LegendaryCrafting.GetRuneforgePowers(nil)
                        local output = {}
                        for _, id in pairs(ids) do
                            local runeforgePower = C_LegendaryCrafting.GetRuneforgePowerInfo(id)
                            if runeforgePower ~= nil then
                                insert(output, id .. ": " .. runeforgePower.name)
                            end
                        end
                        insert(output, "")
                        insert(output, "Equipped:")
                        for id in kpairs(self.equippedRuneforgeById) do
                            insert(output, "    " .. id)
                        end
                        return concat(output, "\n")
                    end
                }
            }
        }
        self.debugLegendaries = {
            type = "group",
            name = "Legendaries",
            args = {
                legendaries = {
                    type = "input",
                    name = "Legendaries",
                    multiline = 25,
                    width = "full",
                    get = function()
                        local output = {}
                        insert(output, "Legendary bonus IDs:")
                        for id in kpairs(self.equippedLegendaryById) do
                            insert(output, "    " .. id)
                        end
                        return concat(output, "\n")
                    end
                }
            }
        }
        self.handleInitialize = function()
            self.module:RegisterMessage("Ovale_EquipmentChanged", self.handleOvaleEquipmentChanged)
        end
        self.handleDisable = function()
            self.module:UnregisterMessage("Ovale_EquipmentChanged")
        end
        self.handleOvaleEquipmentChanged = function(event, slot)
            for id, slotName in pairs(self.equippedLegendaryById) do
                if slotName == slot then
                    self.equippedLegendaryById[id] = nil
                end
            end
            local quality = self.equipment:getEquippedItemQuality(slot)
            if quality == Enum.ItemQuality.Legendary then
                local bonusIds = self.equipment:getEquippedItemBonusIds(slot)
                if #bonusIds > 0 then
                    local id = bonusIds[1]
                    self.equippedLegendaryById[id] = slot
                end
            end
            for id, slotName in pairs(self.equippedRuneforgeById) do
                if slotName == slot then
                    self.equippedRuneforgeById[id] = nil
                end
            end
            local location = self.equipment:getEquippedItemLocation(slot)
            if location ~= nil then
                if C_LegendaryCrafting.IsRuneforgeLegendary(location) then
                    local componentInfo = C_LegendaryCrafting.GetRuneforgeLegendaryComponentInfo(location)
                    local id = tonumber(componentInfo.powerID)
                    self.equippedRuneforgeById[id] = slot
                end
            end
        end
        self.equippedRuneforge = function(positionalParameters)
            local id = positionalParameters[1]
            if self.equippedLegendaryById[id] or self.equippedRuneforgeById[id] then
                return returnBoolean(true)
            end
            return returnBoolean(false)
        end
        debug.defaultOptions.args["runeforge"] = self.debugRuneforges
        debug.defaultOptions.args["legendaries"] = self.debugLegendaries
        self.module = ovale:createModule("OvaleRuneforge", self.handleInitialize, self.handleDisable, aceEvent)
    end,
    registerConditions = function(self, condition)
        condition:registerCondition("equippedruneforge", false, self.equippedRuneforge)
        condition:registerCondition("runeforge", false, self.equippedRuneforge)
    end,
})
